<?php
/*
Plugin Name: Jupyter Notebook Embedder
Description: 在 WordPress 文章中嵌入 Jupyter Notebook HTML 文件
Version: 1.0
Author: Elias Cheung with Trae AI
*/

// 防止直接访问此文件
if (!defined('ABSPATH')) {
    exit;
}

// 创建上传目录
function create_jupyter_upload_dir() {
    $upload_dir = wp_upload_dir();
    $jupyter_dir = $upload_dir['basedir'] . '/jupyter-notebooks';
    if (!file_exists($jupyter_dir)) {
        wp_mkdir_p($jupyter_dir);
    }
}
register_activation_hook(__FILE__, 'create_jupyter_upload_dir');

// 注册短代码
function jupyter_notebook_shortcode($atts) {
    $atts = shortcode_atts(array(
        'notebook' => '', // HTML 文件名称
    ), $atts);
    
    // 获取上传目录信息
    $upload_dir = wp_upload_dir();
    
    // 构建完整的文件路径
    $filename = $atts['notebook'];
    if (!preg_match('/\.html$/', $filename)) {
        $filename .= '.html';
    }
    $full_path = $upload_dir['baseurl'] . '/jupyter-notebooks/' . $filename;
    
    // 从文件名中移除.html用作ID
    $iframe_id = basename($filename, '.html');
    
    // 构建输出
    // 修改iframe输出，添加初始高度和包装器
    $output = sprintf(
        '<div class="jupyter-notebook-wrapper">
            <iframe id="%s" style="border: 1px lightgrey solid; height: 300px;" src="%s" width="100%%" frameborder="0" scrolling="no"></iframe>
            <button class="jupyter-toggle-btn">展开</button>
         </div>',
        esc_attr($iframe_id),
        esc_url($full_path)
    );
    
    // 修改自适应高度脚本
    $output .= sprintf(
        '<script>
        (function(){
            var iframe = document.getElementById("%s");
            var wrapper = iframe.parentElement;
            var toggleBtn = wrapper.querySelector(".jupyter-toggle-btn");
            var isExpanded = false;
            var fullHeight = 0;
            
            function getFullHeight() {
                return iframe.contentWindow.document.body.scrollHeight + 4;
            }
            
            function toggleHeight() {
                if (isExpanded) {
                    iframe.style.height = "300px";
                    toggleBtn.textContent = "展开";
                } else {
                    fullHeight = getFullHeight();
                    iframe.style.height = fullHeight + "px";
                    toggleBtn.textContent = "收回";
                }
                isExpanded = !isExpanded;
            }
            
            toggleBtn.addEventListener("click", toggleHeight);
            
            // 页面大小改变时重新计算
            window.addEventListener("resize", function() {
                if (isExpanded) {
                    fullHeight = getFullHeight();
                    iframe.style.height = fullHeight + "px";
                }
            });
            
            // iframe加载完成后初始化
            iframe.addEventListener("load", function() {
                fullHeight = getFullHeight();
            });
        })();
        </script>',
        esc_attr($iframe_id)
    );
    
    return $output;
}
add_shortcode('jupyter', 'jupyter_notebook_shortcode');

// 允许上传HTML文件
function allow_html_uploads($mime_types) {
    $mime_types['html'] = 'text/html';
    return $mime_types;
}
add_filter('upload_mimes', 'allow_html_uploads');

// 修改上传目录
function modify_jupyter_upload_dir($uploads) {
    if (isset($_POST['name']) && strpos($_POST['name'], '.html') !== false) {
        $uploads['subdir'] = '/jupyter-notebooks';
        $uploads['path'] = $uploads['basedir'] . $uploads['subdir'];
        $uploads['url'] = $uploads['baseurl'] . $uploads['subdir'];
    }
    return $uploads;
}
add_filter('upload_dir', 'modify_jupyter_upload_dir');

// 添加管理菜单
function add_jupyter_admin_menu() {
    add_menu_page(
        'Jupyter Notebooks 管理', // 页面标题
        'Jupyter Notebooks', // 菜单标题
        'manage_options', // 权限
        'jupyter-notebooks', // 菜单slug
        'jupyter_admin_page', // 回调函数
        'dashicons-media-document', // 图标
        99 // 位置
    );
}
add_action('admin_menu', 'add_jupyter_admin_menu');

// 注册后台所需的样式和脚本
function jupyter_admin_enqueue_scripts($hook) {
    if ('toplevel_page_jupyter-notebooks' !== $hook) {
        return;
    }
    
    wp_enqueue_style('jupyter-admin-style', plugins_url('css/admin.css', __FILE__));
    wp_enqueue_script('jupyter-admin-script', plugins_url('js/admin.js', __FILE__), array('jquery'), '1.0', true);
    wp_localize_script('jupyter-admin-script', 'jupyterAjax', array(
        'ajaxurl' => admin_url('admin-ajax.php'),
        'nonce' => wp_create_nonce('jupyter_nonce')
    ));
}
add_action('admin_enqueue_scripts', 'jupyter_admin_enqueue_scripts');

// 管理页面HTML输出
function jupyter_admin_page() {
    include plugin_dir_path(__FILE__) . 'templates/admin-page.php';
}

// 处理文件上传
function handle_jupyter_upload() {
    check_ajax_referer('jupyter_nonce', 'nonce');
    
    if (!current_user_can('manage_options')) {
        wp_send_json_error('权限不足');
    }
    
    $file = $_FILES['notebook'];
    $upload_dir = wp_upload_dir();
    $jupyter_dir = $upload_dir['basedir'] . '/jupyter-notebooks';
    
    // 检查文件类型
    $file_type = wp_check_filetype($file['name'], array('html' => 'text/html'));
    if (!$file_type['type']) {
        wp_send_json_error('只允许上传HTML文件');
    }
    
    // 移动文件到目标目录
    $target_path = $jupyter_dir . '/' . $file['name'];
    if (move_uploaded_file($file['tmp_name'], $target_path)) {
        wp_send_json_success(array(
            'filename' => $file['name'],
            'upload_time' => current_time('mysql')
        ));
    } else {
        wp_send_json_error('文件上传失败');
    }
}
add_action('wp_ajax_jupyter_upload', 'handle_jupyter_upload');

// 处理文件删除
function handle_jupyter_delete() {
    check_ajax_referer('jupyter_nonce', 'nonce');
    
    if (!current_user_can('manage_options')) {
        wp_send_json_error('权限不足');
    }
    
    $filename = sanitize_text_field($_POST['filename']);
    $upload_dir = wp_upload_dir();
    $file_path = $upload_dir['basedir'] . '/jupyter-notebooks/' . $filename;
    
    if (unlink($file_path)) {
        wp_send_json_success();
    } else {
        wp_send_json_error('删除失败');
    }
}
add_action('wp_ajax_jupyter_delete', 'handle_jupyter_delete');

// 处理文件下载
function handle_jupyter_download() {
    if (!current_user_can('manage_options')) {
        wp_die('权限不足');
    }
    
    $filename = sanitize_text_field($_GET['filename']);
    $upload_dir = wp_upload_dir();
    $file_path = $upload_dir['basedir'] . '/jupyter-notebooks/' . $filename;
    
    if (file_exists($file_path)) {
        header('Content-Type: text/html');
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        header('Content-Length: ' . filesize($file_path));
        readfile($file_path);
        exit;
    }
    wp_die('文件不存在');
}
add_action('admin_post_jupyter_download', 'handle_jupyter_download');